/* ======================================================================= */
/*  TEXAS INSTRUMENTS, INC.                                                */
/*                                                                         */
/*  FFTLIB  FFT Library                                                    */
/*                                                                         */
/* Copyright (C) 2013 Texas Instruments Incorporated - http://www.ti.com/  */ 
/*                                                                         */
/*                                                                         */
/*  Redistribution and use in source and binary forms, with or without     */
/*  modification, are permitted provided that the following conditions     */
/*  are met:                                                               */
/*                                                                         */
/*    Redistributions of source code must retain the above copyright       */
/*    notice, this list of conditions and the following disclaimer.        */
/*                                                                         */
/*    Redistributions in binary form must reproduce the above copyright    */
/*    notice, this list of conditions and the following disclaimer in the  */
/*    documentation and/or other materials provided with the               */
/*    distribution.                                                        */
/*                                                                         */
/*    Neither the name of Texas Instruments Incorporated nor the names of  */
/*    its contributors may be used to endorse or promote products derived  */
/*    from this software without specific prior written permission.        */
/*                                                                         */
/*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS    */
/*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT      */
/*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR  */
/*  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT   */
/*  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,  */
/*  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT       */
/*  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,  */
/*  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY  */
/*  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT    */
/*  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE  */
/*  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.   */
/*                                                                         */
/* ======================================================================= */

#include <xdc/std.h>
#include "c6x.h"
#include "stdlib.h"
#include <ti/fftlib/src/common/fft/fft_dp_mixed_bs.h>
#include "fft_dp_1d_c2c_batch.h"
#include <ti/fftlib/src/common/fft_edma.h>
#include <ti/dsplib/src/DSPF_dp_fftDPxDP/DSPF_dp_fftDPxDP.h>

int fft_dp_1d_c2c_batch_direct (
    /* int   N;        FFT size         */
    /* int   batch;    batch size       */
    /* int   rad;      radix            */
    /* double *in;      input pointer    */
    /* double *out;     output pointer   */
    /* double *tw;      twiddle pointer  */
    fft_param_u u,
    void  *edmaState
)
{
    int   error = FFT_NOERR, i;
    /* input parameters */
    int     N       = u.dp_1d_c2c_batch_e.N;       /* signal size     */
    int     M       = u.dp_1d_c2c_batch_e.M;       /* FFT size        */
    int     batch   = u.dp_1d_c2c_batch_e.batch;   /* batch size      */
    int     rad     = u.dp_1d_c2c_batch_e.rad;     /* radix           */
    int     N_p2    = u.dp_1d_c2c_batch_e.N_p2;    /* power of 2 size in mixed radix */
    int     s_r3    = u.dp_1d_c2c_batch_e.s_r3;    /* number stages of radix 3       */
    int     s_r5    = u.dp_1d_c2c_batch_e.s_r5;    /* number stages of radix 5       */
    double *in      = u.dp_1d_c2c_batch_e.in;      /* input pointer   */
    double *out     = u.dp_1d_c2c_batch_e.out;     /* output pointer  */
    double *tw      = u.dp_1d_c2c_batch_e.tw;      /* twiddle pointer */
    double *Bn      = u.dp_1d_c2c_batch_e.Bn;
    double *workbuf = u.dp_1d_c2c_batch_e.workbuf;

    for (i = 0; i < batch; i++) {
      if (N == M) {
        if (s_r3 == 0 && s_r5 == 0) {
          DSPF_dp_fftDPxDP(N, &in[i*2*N], tw, &out[i*2*N], rad, 0, N);
        }
        else {
          fft_DPxDP_mixed_radix (N, &in[i*2*N], tw, &out[i*2*N], N_p2, s_r3, s_r5, rad);
        }
      }
      else
      {
        fft_DPxDP_bs (N, M, rad, &in[i*2*N], &out[i*2*N], tw, Bn, workbuf);
      }
    }
    return error;
}

int fft_dp_1d_c2c_batch_ecpy (
    /* int     N;       FFT size         */
    /* int     batch;   batch size       */
    /* int     rad;     radix            */
    /* double *in;      input pointer    */
    /* double *out;     output pointer   */
    /* double *in_w;    input pointer    */
    /* double *out_w;   output pointer   */
    /* double *tw;      twiddle pointer  */
    fft_param_u u,
    void *edmaState
)
{
    /* input parameters */
    int     N       = u.dp_1d_c2c_batch_e.N;       /* signal size                    */
    int     M       = u.dp_1d_c2c_batch_e.M;       /* FFT size                       */
    int     batch   = u.dp_1d_c2c_batch_e.batch;   /* batch size                     */
    int     rad     = u.dp_1d_c2c_batch_e.rad;     /* radix                          */
    int     twsize  = u.dp_1d_c2c_batch_e.twsize;  /* twiddle size                   */
    int     N_p2    = u.dp_1d_c2c_batch_e.N_p2;    /* power of 2 size in mixed radix */
    int     s_r3    = u.dp_1d_c2c_batch_e.s_r3;    /* number stages of radix 3       */
    int     s_r5    = u.dp_1d_c2c_batch_e.s_r5;    /* number stages of radix 5       */
    double *in      = u.dp_1d_c2c_batch_e.in;      /* input pointer                  */
    double *out     = u.dp_1d_c2c_batch_e.out;     /* output pointer                 */
    double *in_w    = u.dp_1d_c2c_batch_e.in_w;    /* input working buffer pointer   */
    double *out_w   = u.dp_1d_c2c_batch_e.out_w;   /* output working buffer pointer  */
    double *tw      = u.dp_1d_c2c_batch_e.tw;      /* twiddle pointer                */
    double *tw_w    = u.dp_1d_c2c_batch_e.tw_w;    /* twiddle working buffer pointer */
    double *Bn      = u.dp_1d_c2c_batch_e.Bn;
    double *Bn_w    = u.dp_1d_c2c_batch_e.Bn_w;
    double *workbuf = u.dp_1d_c2c_batch_e.workbuf;

    int   error = FFT_NOERR;
    int   j,ch_i,ch_o,ch_i1,ch_o1,dum0,dum1;
    int   insize  = N*2*sizeof(double),
          outsize = N*2*sizeof(double),
          Bnsize  = (2*M)*sizeof(double);

    double *restrict in_ptr = in, *restrict out_ptr = out;
    double *restrict input, *restrict output, *restrict in1, *restrict out1, *dum2, *dum3;
    FFT_EDMA_Struct  *edma = (FFT_EDMA_Struct *)edmaState;

    if (tw_w != NULL) {
      /* prepare twiddle buffer */
      EdmaMgr_copy1D1D(
        edma->channel[FFT_XFER_CHANNEL3], 
        (void *)tw,
        (void *)tw_w,
        twsize);
      tw = tw_w;
    }

    if (Bn_w != NULL) {
      /* prepare Bn buffer */
      EdmaMgr_copy1D1D(
        edma->channel[FFT_XFER_CHANNEL1],
        (void *)Bn,
        (void *)Bn_w,
        Bnsize);
      Bn = Bn_w;
    }
    /* prepare first FFT input buffer */
    EdmaMgr_copy1D1D( 
      edma->channel[FFT_XFER_CHANNEL0], 
      (void *)in_ptr,
      (void *)in_w,
      insize);
    in_ptr += 2*N;

    /* set pingpong buffer pointers */
    in1    = in_w;
    out1   = out_w;
    input  = &in_w[2*N];
    output = &out_w[2*N];

    /* prepare second FFT input buffer */
    EdmaMgr_copy1D1D( 
      edma->channel[FFT_XFER_CHANNEL2],
      (void *)in_ptr,
      (void *)input,
      insize);
    in_ptr += 2*N;

    /* wait for first FFT input */
    EdmaMgr_wait(edma->channel[FFT_XFER_CHANNEL0]);

    if (tw_w != NULL) {
      EdmaMgr_wait(edma->channel[FFT_XFER_CHANNEL3]);
    }

    /* first FFT */
    if (N == M) {
      if (s_r3 == 0 && s_r5 == 0) {
        DSPF_dp_fftDPxDP (N, in1, tw, out1, rad, 0, N);
      }
      else {
        fft_DPxDP_mixed_radix (N, in1, tw, out1, N_p2, s_r3, s_r5, rad);
      }
    }
    else
    {
      if (Bn_w != NULL) {
      EdmaMgr_wait(edma->channel[FFT_XFER_CHANNEL1]);
      }
      fft_DPxDP_bs (N, M, rad, in1, out1, tw, Bn, workbuf);
    }

    /* export output data of first FFT */
    EdmaMgr_copy1D1D( 
      edma->channel[FFT_XFER_CHANNEL1], 
      (void *)out1,
      (void *)out_ptr,
      outsize);
    out_ptr += 2*N;

    /* prepare third FFT input buffer */
    EdmaMgr_copy1D1D( 
      edma->channel[FFT_XFER_CHANNEL0], 
      (void *)in_ptr,
      (void *)in_w,
      insize);
    in_ptr += 2*N;

    /* set channel numbers */
    ch_i  = FFT_XFER_CHANNEL2;
    ch_o  = FFT_XFER_CHANNEL3;
    ch_i1 = FFT_XFER_CHANNEL0;
    ch_o1 = FFT_XFER_CHANNEL1;

    /* wait for second FFT input */
    EdmaMgr_wait(edma->channel[FFT_XFER_CHANNEL2]);

    _nassert (batch >= 4);
    for (j = 1; j < batch-2; j++) {

      if (N == M) {
        if (s_r3 == 0 && s_r5 == 0) {
          DSPF_dp_fftDPxDP (N, input, tw, output, rad, 0, N);
        }
        else {
          fft_DPxDP_mixed_radix (N, input, tw, output, N_p2, s_r3, s_r5, rad);
        }
      }
      else
      {
        fft_DPxDP_bs (N, M, rad, input, output, tw, Bn, workbuf);
      }

      /* export output data */
      EdmaMgr_copy1D1D( 
        edma->channel[ch_o],
        (void *)output,
        (void *)out_ptr,
        outsize);
      out_ptr += 2*N;

      /* prepare input data */
      EdmaMgr_copy1D1D( 
        edma->channel[ch_i],
        (void *)in_ptr,
        (void *)input,
        insize);
      in_ptr += 2*N;

      /* swap pingpong buffer pointers and XFER channel */
      dum2   = input;
      input  = in1;
      in1    = dum2;
      dum3   = output;
      output = out1;
      out1   = dum3;

      dum0   = ch_i;
      ch_i   = ch_i1;
      ch_i1  = dum0;
      dum1   = ch_o;
      ch_o   = ch_o1;
      ch_o1  = dum1;

      /* wait for EDMA */
      EdmaMgr_wait(edma->channel[ch_i]);
      EdmaMgr_wait(edma->channel[ch_o]);
    }

    /* second to last FFT */
    if (N == M) {
      if (s_r3 == 0 && s_r5 == 0) {
        DSPF_dp_fftDPxDP (N, input, tw, output, rad, 0, N);
      }
      else {
        fft_DPxDP_mixed_radix (N, input, tw, output, N_p2, s_r3, s_r5, rad);
      }
    }
    else
    {
      fft_DPxDP_bs (N, M, rad, input, output, tw, Bn, workbuf);
    }

    /* export output data */
    EdmaMgr_copy1D1D( 
      edma->channel[ch_o],
      (void *)output,
      (void *)out_ptr,
      outsize);
    out_ptr += 2*N;

    /* last FFT */
    if (N == M) {
      if (s_r3 == 0 && s_r5 == 0) {
        DSPF_dp_fftDPxDP (N, in1, tw, out1, rad, 0, N);
      }
      else {
        fft_DPxDP_mixed_radix (N, in1, tw, out1, N_p2, s_r3, s_r5, rad);
      }
    }
    else
    {
      fft_DPxDP_bs (N, M, rad, in1, out1, tw, Bn, workbuf);
    }

    /* export out1 data */
    EdmaMgr_copy1D1D( 
      edma->channel[ch_o1],
      (void *)out1,
      (void *)out_ptr,
      outsize);

    /* wait for last two output EDMA */
    EdmaMgr_wait(edma->channel[ch_o ]);
    EdmaMgr_wait(edma->channel[ch_o1]);

    return error;
} /* fft_dp_1d_c2c_batch_ecpy */

/* ======================================================================== */
/*  End of file:  fft_dp_1d_c2c_batch.c                                     */
/* ------------------------------------------------------------------------ */
/*            Copyright (c) 2013 Texas Instruments, Incorporated.           */
/*                           All Rights Reserved.                           */
/* ======================================================================== */
