/* ======================================================================= */
/*  TEXAS INSTRUMENTS, INC.                                                */
/*                                                                         */
/*  FFTLIB  FFT Library                                                    */
/*                                                                         */
/* Copyright (C) 2013 Texas Instruments Incorporated - http://www.ti.com/  */ 
/*                                                                         */
/*                                                                         */
/*  Redistribution and use in source and binary forms, with or without     */
/*  modification, are permitted provided that the following conditions     */
/*  are met:                                                               */
/*                                                                         */
/*    Redistributions of source code must retain the above copyright       */
/*    notice, this list of conditions and the following disclaimer.        */
/*                                                                         */
/*    Redistributions in binary form must reproduce the above copyright    */
/*    notice, this list of conditions and the following disclaimer in the  */
/*    documentation and/or other materials provided with the               */
/*    distribution.                                                        */
/*                                                                         */
/*    Neither the name of Texas Instruments Incorporated nor the names of  */
/*    its contributors may be used to endorse or promote products derived  */
/*    from this software without specific prior written permission.        */
/*                                                                         */
/*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS    */
/*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT      */
/*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR  */
/*  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT   */
/*  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,  */
/*  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT       */
/*  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,  */
/*  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY  */
/*  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT    */
/*  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE  */
/*  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.   */
/*                                                                         */
/* ======================================================================= */

#include <math.h>
#include <stdio.h>
#include <c6x.h>
#include <ti/dsplib/src/DSPF_dp_fftDPxDP/DSPF_dp_fftDPxDP.h>
#include "fft_dp_1d_r2c_batch.h"
#include <ti/csl/csl_cacheAux.h>

/* FFT 1d dp r2c batch memory buffers */
#define FFT_DP_1D_R2C_BATCH_EXTERN_X_BUFN      0
#define FFT_DP_1D_R2C_BATCH_EXTERN_Y_BUFN      (FFT_DP_1D_R2C_BATCH_EXTERN_X_BUFN     +1)
#define FFT_DP_1D_R2C_BATCH_EXTERN_W_BUFN      (FFT_DP_1D_R2C_BATCH_EXTERN_Y_BUFN     +1)
#define FFT_DP_1D_R2C_BATCH_EXTERN_BN_BUFN     (FFT_DP_1D_R2C_BATCH_EXTERN_W_BUFN     +1)
#define FFT_DP_1D_R2C_BATCH_INTERN_X_BUFN      (FFT_DP_1D_R2C_BATCH_EXTERN_BN_BUFN    +1)
#define FFT_DP_1D_R2C_BATCH_INTERN_Y_BUFN      (FFT_DP_1D_R2C_BATCH_INTERN_X_BUFN     +1)
#define FFT_DP_1D_R2C_BATCH_INTERN_W_BUFN      (FFT_DP_1D_R2C_BATCH_INTERN_Y_BUFN     +1)
#define FFT_DP_1D_R2C_BATCH_INTERN_BN_BUFN     (FFT_DP_1D_R2C_BATCH_INTERN_W_BUFN     +1)
#define FFT_DP_1D_R2C_BATCH_INTERN_WORK_BUFN   (FFT_DP_1D_R2C_BATCH_INTERN_BN_BUFN    +1)
#define FFT_DP_1D_R2C_BATCH_NBUFS              (FFT_DP_1D_R2C_BATCH_INTERN_WORK_BUFN  +1)

/* class, log2align, size, base */
static FFTmemBuffer_t FFT_dp_1d_r2c_batch_Bufs[FFT_DP_1D_R2C_BATCH_NBUFS] = {
  {FFT_MEM_CLASS_EXTERNAL, 3, 0, 0},   /* EXTERN_X */
  {FFT_MEM_CLASS_EXTERNAL, 3, 0, 0},   /* EXTERN_Y */
  {FFT_MEM_CLASS_EXTERNAL, 3, 0, 0},   /* EXTERN_W */
  {FFT_MEM_CLASS_EXTERNAL, 3, 0, 0},   /* EXTERN_BN */
  {FFT_MEM_CLASS_INTERNAL, 3, 0, 0},   /* INTERN_X */
  {FFT_MEM_CLASS_INTERNAL, 3, 0, 0},   /* INTERN_Y */
  {FFT_MEM_CLASS_INTERNAL, 3, 0, 0},   /* INTERN_W */
  {FFT_MEM_CLASS_INTERNAL, 3, 0, 0},   /* INTERN_BN */
  {FFT_MEM_CLASS_INTERNAL, 3, 0, 0}    /* INTERN_WORK */
};

fft_plan_t fft_dp_plan_1d_r2c_batch (
    int   N,        /* FFT size */
    int   batch,    /* batch size */
    int   mode,     /* FFT_DIRECT or FFT_ECPY */
    fft_callout_t fxns
)
{
    int          i, j, k, rad, use_bs = 1, n1, N1, dum, N_p2, s_r3 = 0, s_r5 = 0;
    double      *tw;
    double      *restrict Bk, *restrict bn, *restrict Bn, *restrict work;
    fft_plan_t   p;
    const double PI = 3.14159265358979323846;

    if (mode == FFT_DIRECT) {
      p.fftfxn = fft_dp_1d_r2c_batch_direct;
    }
    else {  /* FFT_ECPY */
      p.fftfxn = fft_dp_1d_r2c_batch_ecpy;
    }

    p.nbuf = FFT_DP_1D_R2C_BATCH_NBUFS;
    memcpy(&p.fftcout, &fxns, sizeof(fft_callout_t));
    memcpy(p.bufs, FFT_dp_1d_r2c_batch_Bufs, FFT_DP_1D_R2C_BATCH_NBUFS*sizeof(FFTmemBuffer_t));

    p.u.dp_1d_r2c_batch_e.N     = N;
    p.u.dp_1d_r2c_batch_e.batch = batch;
    N_p2                        = N/2;
    p.u.dp_1d_r2c_batch_e.N_p2  = N_p2;
    p.u.dp_1d_r2c_batch_e.s_r3  = 0;
    p.u.dp_1d_r2c_batch_e.s_r5  = 0;

    /* calculate the size for zero padding when the signal size is not power of 2 */
    n1 = (N<<1)-1;
    j = _norm(n1);
    N1 = 1 << (31-j);

    /* if N is odd, bypass the rest of calculation, use Bluestein */
    if (((N>>1)<<1) == N)
    {
      /* if N is power of 2, no zero padding, regular radix 2 or 4 fft */
      if (N == (N1>>1))
      { /* power of 2 */
        N1 = N;
        j = j+1;
        use_bs = 0;
      }
      else {
        dum = N/2;
        while (dum/3*3 == dum) {
          s_r3++;
          dum /= 3;
        }
        while (dum/5*5 == dum) {
          s_r5++;
          dum /= 5;
        }
        if ((dum >= 4) && (dum == (1 << (30-_norm(dum))))) {
        /* mixed radix 2 and 3 */
          N_p2                       = dum;
          p.u.dp_1d_r2c_batch_e.N_p2 = N_p2;
          p.u.dp_1d_r2c_batch_e.s_r3 = s_r3;
          p.u.dp_1d_r2c_batch_e.s_r5 = s_r5;
          use_bs = 0;
          N1 = N;
          j  = _norm(dum);
        }
      }
    }

    /* calculate radix for radix 2 or 4 c2c fft */
    if (j%2 == use_bs)
      rad = 4;
    else
      rad = 2;

    p.u.dp_1d_r2c_batch_e.rad = rad;
    p.u.dp_1d_r2c_batch_e.M   = N1;

    /* calculate memory requirements */
    p.bufs[FFT_DP_1D_R2C_BATCH_EXTERN_X_BUFN].size = sizeof(double)*batch*N;
    p.bufs[FFT_DP_1D_R2C_BATCH_EXTERN_Y_BUFN].size = sizeof(double)*batch*N*2;
    p.bufs[FFT_DP_1D_R2C_BATCH_INTERN_X_BUFN].size = sizeof(double)*N*2;
    p.bufs[FFT_DP_1D_R2C_BATCH_INTERN_Y_BUFN].size = sizeof(double)*N*4;


    if (use_bs == 1)
    { /* when size is not power of 2 and not multiple of 3&5 */
      p.u.dp_1d_r2c_batch_e.twsize = sizeof(double)*(2*N1+2*N);
      p.bufs[FFT_DP_1D_R2C_BATCH_EXTERN_W_BUFN].size    = p.u.dp_1d_r2c_batch_e.twsize;
      p.bufs[FFT_DP_1D_R2C_BATCH_INTERN_W_BUFN].size    = p.u.dp_1d_r2c_batch_e.twsize;
      p.bufs[FFT_DP_1D_R2C_BATCH_EXTERN_BN_BUFN].size   = sizeof(double)*(2*N1);
      p.bufs[FFT_DP_1D_R2C_BATCH_INTERN_BN_BUFN].size   = sizeof(double)*(2*N1);
      p.bufs[FFT_DP_1D_R2C_BATCH_INTERN_WORK_BUFN].size = sizeof(double)*(4*N1);
    }
    else
    {
      if ((s_r3 == 0) && (s_r5 == 0)) {
        p.u.dp_1d_r2c_batch_e.twsize = sizeof(double)*N*3/2;
        p.bufs[FFT_DP_1D_R2C_BATCH_EXTERN_W_BUFN].size = p.u.dp_1d_r2c_batch_e.twsize;
        p.bufs[FFT_DP_1D_R2C_BATCH_INTERN_W_BUFN].size = p.u.dp_1d_r2c_batch_e.twsize;
      }
      else {
        dum = 0;
        n1  = 2*N;
        for (i = 0; i < s_r3; i++) {
          dum += n1/3;
          n1  /= 3;
        }
        n1 *= 2;  /* radix-5 has 8 twiddle vs radix-3 4 twiddles */        
        for (i = 0; i < s_r5; i++) {
          dum += n1/5;
          n1  /= 5;
        }
        p.u.dp_1d_r2c_batch_e.twsize = sizeof(double)*(N/2+2*N_p2+dum);
        p.bufs[FFT_DP_1D_R2C_BATCH_EXTERN_W_BUFN].size = p.u.dp_1d_r2c_batch_e.twsize;
        p.bufs[FFT_DP_1D_R2C_BATCH_INTERN_W_BUFN].size = p.u.dp_1d_r2c_batch_e.twsize;
      }
      p.bufs[FFT_DP_1D_R2C_BATCH_EXTERN_BN_BUFN].size   = 0;
      p.bufs[FFT_DP_1D_R2C_BATCH_INTERN_BN_BUFN].size   = 0;
      p.bufs[FFT_DP_1D_R2C_BATCH_INTERN_WORK_BUFN].size = 0;
    }

    (*fxns.memoryRequest)(FFT_DP_1D_R2C_BATCH_NBUFS, p.bufs);
    
    p.u.dp_1d_r2c_batch_e.in    = p.bufs[FFT_DP_1D_R2C_BATCH_EXTERN_X_BUFN].base;
    p.u.dp_1d_r2c_batch_e.out   = p.bufs[FFT_DP_1D_R2C_BATCH_EXTERN_Y_BUFN].base;
    p.u.dp_1d_r2c_batch_e.in_w  = p.bufs[FFT_DP_1D_R2C_BATCH_INTERN_X_BUFN].base;
    p.u.dp_1d_r2c_batch_e.out_w = p.bufs[FFT_DP_1D_R2C_BATCH_INTERN_Y_BUFN].base;
    p.u.dp_1d_r2c_batch_e.tw_w  = p.bufs[FFT_DP_1D_R2C_BATCH_INTERN_W_BUFN].base;
    tw                          = p.bufs[FFT_DP_1D_R2C_BATCH_EXTERN_W_BUFN].base;
    p.u.dp_1d_r2c_batch_e.tw    = tw;

    if (use_bs == 1)
    {
      /* if use Bluestein FFT, set the buffers to do Bluestein */
      p.u.dp_1d_r2c_batch_e.Bn      = p.bufs[FFT_DP_1D_R2C_BATCH_EXTERN_BN_BUFN].base;
      p.u.dp_1d_r2c_batch_e.Bn_w    = p.bufs[FFT_DP_1D_R2C_BATCH_INTERN_BN_BUFN].base;
      p.u.dp_1d_r2c_batch_e.workbuf = p.bufs[FFT_DP_1D_R2C_BATCH_INTERN_WORK_BUFN].base;
    }
    else
    {
      /* otherwise, set the buffers to NULL */
      p.u.dp_1d_r2c_batch_e.Bn      = NULL;
      p.u.dp_1d_r2c_batch_e.Bn_w    = NULL;
      p.u.dp_1d_r2c_batch_e.workbuf = NULL;
    }

    /* twiddle generation */
    k = 0;

    if (use_bs == 0)
    {
      for (i = 1; i < N >> 2; i++)
      {
          tw[k]     = sin (2 * PI * i / N) / 2;
          tw[k + 1] = cos (2 * PI * i / N) / 2;
          k += 2;
      }

      k = N/2;
      for (j = 1; j <= N_p2 >> 2; j = j << 2)
      {
        for (i = 0; i < N_p2 >> 2; i += j)
        {
           tw[k]     = cos (2 * PI * i / N_p2);
           tw[k + 1] = sin (2 * PI * i / N_p2);
           tw[k + 2] = cos (4 * PI * i / N_p2);
           tw[k + 3] = sin (4 * PI * i / N_p2);
           tw[k + 4] = cos (6 * PI * i / N_p2);
           tw[k + 5] = sin (6 * PI * i / N_p2);
           k += 6;
        }
      }

      if (s_r3 != 0 || s_r5 != 0) {
        /* generate twiddles for radix-3 loop */
        k  = N/2+N_p2*2;
        n1 = N/2;
        for (i = 0; i < s_r3; i++) {
          for (j = 0; j < n1/3; j++) {
            tw[k]     = cos (2 * PI * j / n1);
            tw[k + 1] = sin (2 * PI * j / n1);
            tw[k + 2] = cos (4 * PI * j / n1);
            tw[k + 3] = sin (4 * PI * j / n1);
            k += 4;
          }
          n1 /= 3;
        }
        /* generate twiddles for radix-5 loop */
        for (i = 0; i < s_r5; i++) {
          for (j = 0; j < n1/5; j++) {
            tw[k]     = cos (2 * PI * j / n1);
            tw[k + 1] = sin (2 * PI * j / n1);
            tw[k + 2] = cos (4 * PI * j / n1);
            tw[k + 3] = sin (4 * PI * j / n1);
            tw[k + 4] = cos (6 * PI * j / n1);
            tw[k + 5] = sin (6 * PI * j / n1);
            tw[k + 6] = cos (8 * PI * j / n1);
            tw[k + 7] = sin (8 * PI * j / n1);
            k += 8;
          }
          n1 /= 5;
        }
      }
    }
    else
    { /* generate twiddle for c2c fft */

      /*--------------------------------------
       * TW is organized as follows:
       *   twiddle for fft and ifft, size: 2*N1
       *   Bk,                       size: 2*N
       *-------------------------------------*/

      k = 0;

      for (j = 1; j <= N1 >> 2; j = j << 2) {
        for (i = 0; i < N1 >> 2; i += j) {
          tw[k]     =  cos (2 * PI * i / N1);
          tw[k + 1] =  sin (2 * PI * i / N1);
          tw[k + 2] =  cos (4 * PI * i / N1);
          tw[k + 3] =  sin (4 * PI * i / N1);
          tw[k + 4] =  cos (6 * PI * i / N1);
          tw[k + 5] =  sin (6 * PI * i / N1);
          k += 6;
        }
      }
      k = 2*N1;

      /* phase factor generation */
      for (i = 0; i < N ; i++) {
        tw[k]     = cos (PI * i * i / N);
        tw[k + 1] = sin (PI * i * i / N);

        k += 2;
      }

      Bn   = p.u.dp_1d_r2c_batch_e.Bn;
      work = p.u.dp_1d_r2c_batch_e.workbuf;

      /* precalculate FFT of bn */

#pragma MUST_ITERATE(4,,4)
      /* first reset the work buffer to zeros */
      for (i = 0; i < 2*N1; i+=1)
      {
        work[i] = 0;
      }

      /* construct bn in work buffer */
      bn = work;
      Bk = tw + 2*N1;

      /*---------------------------------------------*
       *         bn(0)  = Bk(0)
       *         bn(i)  = Bk(i)            0 < i < N
       *         bn(i)  = 0                N <= i < N1-N-1
       *         bn(i)  = Bk(N1-i)    N1-N-1 <= i < N1
       *  Note: bn(i) = Re(bn(i)) + i* Im(bn(i));
       ------------------------------------------- */
      bn[0] = Bk[0];
      bn[1] = Bk[1];

      for(i = 1; i < N; i++)
      {
        bn[2*i]   = bn[2*(N1-i)]   = Bk[2*i];
        bn[2*i+1] = bn[2*(N1-i)+1] = Bk[2*i+1];
      }

      /* FFT of the constructed signal Bn = fft(bn) */
      DSPF_dp_fftDPxDP (N1, bn, tw, Bn, rad, 0, N1);
    }
    CACHE_wbInvAllL2(CACHE_WAIT);        
    return (p);
}

/* ======================================================================== */
/*  End of file:  fft_plan_dp_1d_r2c_batch.c                                */
/* ------------------------------------------------------------------------ */
/*            Copyright (c) 2013 Texas Instruments, Incorporated.           */
/*                           All Rights Reserved.                           */
/* ======================================================================== */
