/* ======================================================================= */
/*  TEXAS INSTRUMENTS, INC.                                                */
/*                                                                         */
/*  FFTLIB  FFT Library                                                    */
/*                                                                         */
/* Copyright (C) 2013 Texas Instruments Incorporated - http://www.ti.com/  */
/*                                                                         */
/*                                                                         */
/*  Redistribution and use in source and binary forms, with or without     */
/*  modification, are permitted provided that the following conditions     */
/*  are met:                                                               */
/*                                                                         */
/*    Redistributions of source code must retain the above copyright       */
/*    notice, this list of conditions and the following disclaimer.        */
/*                                                                         */
/*    Redistributions in binary form must reproduce the above copyright    */
/*    notice, this list of conditions and the following disclaimer in the  */
/*    documentation and/or other materials provided with the               */
/*    distribution.                                                        */
/*                                                                         */
/*    Neither the name of Texas Instruments Incorporated nor the names of  */
/*    its contributors may be used to endorse or promote products derived  */
/*    from this software without specific prior written permission.        */
/*                                                                         */
/*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS    */
/*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT      */
/*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR  */
/*  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT   */
/*  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,  */
/*  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT       */
/*  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,  */
/*  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY  */
/*  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT    */
/*  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE  */
/*  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.   */
/*                                                                         */
/* ======================================================================= */

#include <math.h>
#include <stdio.h>
#include <c6x.h>
#include <ti/csl/csl_cacheAux.h>
#include "fft_omp_dp_1d_c2c.h"

/* FFT 1d dp c2c memory buffers */
#define FFT_DP_1D_C2C_EXTERN_X_BUFN      0
#define FFT_DP_1D_C2C_EXTERN_Y_BUFN      (FFT_DP_1D_C2C_EXTERN_X_BUFN  +1)
#define FFT_DP_1D_C2C_EXTERN_W_BUFN      (FFT_DP_1D_C2C_EXTERN_Y_BUFN  +1)
#define FFT_DP_1D_C2C_INTERN_X_BUFN      (FFT_DP_1D_C2C_EXTERN_W_BUFN  +1)
#define FFT_DP_1D_C2C_INTERN_Y_BUFN      (FFT_DP_1D_C2C_INTERN_X_BUFN  +1)
#define FFT_DP_1D_C2C_INTERN_W_BUFN      (FFT_DP_1D_C2C_INTERN_Y_BUFN  +1)
#define FFT_DP_1D_C2C_INTERN_T_BUFN      (FFT_DP_1D_C2C_INTERN_W_BUFN  +1)
#define FFT_DP_1D_C2C_NBUFS              (FFT_DP_1D_C2C_INTERN_T_BUFN  +1)

/* class, log2align, size, base */
static FFTmemBuffer_t FFT_dp_1d_c2c_Bufs[FFT_DP_1D_C2C_NBUFS] = {
  {FFT_MEM_CLASS_EXTERNAL, 3, 0, 0},   /* EXTERN_X */
  {FFT_MEM_CLASS_EXTERNAL, 3, 0, 0},   /* EXTERN_Y */
  {FFT_MEM_CLASS_EXTERNAL, 3, 0, 0},   /* EXTERN_W */
  {FFT_MEM_CLASS_INTERNAL, 3, 0, 0},   /* INTERN_X */
  {FFT_MEM_CLASS_INTERNAL, 3, 0, 0},   /* INTERN_Y */
  {FFT_MEM_CLASS_INTERNAL, 3, 0, 0},   /* INTERN_W */
  {FFT_MEM_CLASS_INTERNAL, 3, 0, 0}    /* INTERN_T */
};

/* calculate radix for 1D array */
static void calculate_rad (int N, int *n1, int *n2, fft_para_mix_bs_t *fft_para1, 
                    fft_para_mix_bs_t *fft_para2, int *use_bs)
{
  int i, j, rad;
  int s_r3, s_r5, s_r3_2, s_r5_2, dum;

  *use_bs = 1;

  s_r3 = 0;
  s_r5 = 0;

  dum = N;
  
  while (dum/3*3 == dum) {
    s_r3++;
    dum /= 3;
  }
  s_r3_2 = s_r3/2;
  s_r3   = s_r3 - s_r3_2;

  while (dum/5*5 == dum) {
    s_r5++;
    dum /= 5;
  }
  s_r5_2 = s_r5/2;
  s_r5   = s_r5 - s_r5_2;
  
  j = 30 - _norm(dum);
  if ((dum >= 32) && (dum == (1 << j))) {
    *use_bs = 0;
    
    *n1 = 1 << (j-j/2);
    fft_para1->N_p2 = *n1;
    for (i = 0; i < s_r3; i++)
      *n1 *= 3;
    for (i = 0; i < s_r5; i++)
      *n1 *= 5;
    
    *n2 = 1 << (j/2);
    fft_para2->N_p2 = *n2;
    for (i = 0; i < s_r3_2; i++)
      *n2 *= 3;
    for (i = 0; i < s_r5_2; i++)
      *n2 *= 5;

    j = _norm(fft_para1->N_p2);
    if (j%2 == 0)
      rad = 4;
    else
      rad = 2;

    fft_para1->s_r3 = s_r3;
    fft_para1->s_r5 = s_r5;
    fft_para1->rad  = rad;

    j = _norm(fft_para2->N_p2);
    if (j%2 == 0)
      rad = 4;
    else
      rad = 2;
    
    fft_para2->s_r3 = s_r3_2;
    fft_para2->s_r5 = s_r5_2;
    fft_para2->rad  = rad;
  } else {
    //not supported, will trigger a plan failure at this point
    return;
  }

} /*calculate_rad ()*/

/* calculate twiddle size, Bn buffer size and workbuffer size */
static void calculate_mem_size(int N, int s_r3, int s_r5, int N_p2, int *twsize)
{
  int dum, n, i;

  /* Please note that Bn buffer and Workbuf are only used when doing Bluestein
   * meaning when use_bs == 1 */
    if ((s_r3 == 0) && (s_r5 == 0)) {
    /* When signal is power of 2 */
      *twsize   = N*2;
    }
    else {
    /* When signal is power of 3 or 5 */
      dum = 0;
      n  = 4*N;
      for (i = 0; i < s_r3; i++) {
        dum += n/3;
        n  /= 3;
      }
      n *= 2;  /* radix-5 has 8 twiddle vs radix-3 4 twiddles */
      for (i = 0; i < s_r5; i++) {
        dum += n/5;
        n  /= 5;
      }
      *twsize = (2*N_p2+dum);
    }
} /* calculate_mem_size()*/

/* twiddle generation */
static void twiddle_gen (int dim, int N, int n, int rad, int s_r3, int s_r5, int N_p2, double *tw)
{
	int     k, j, i;
  const double PI = 3.14159265358979323846;

  k = 0;
  
  /* twiddle_gen for ecpy - bluestein not supported */
  if (dim == 2) {
    /* global twiddle generation */
    for (i = 0; i < n; i++) {
      tw[k    ] =  cos (2 * PI * i / N);
      tw[k + 1] = -sin (2 * PI * i / N);
      k += 2;
    }
    k = 2*n;
  }
  
  for (j = 1; j <= N_p2 >> 2; j = j << 2) {
      for (i = 0; i < N_p2 >> 2; i += j) {
          tw[k]     = cos (2 * PI * i / N_p2);
          tw[k + 1] = sin (2 * PI * i / N_p2);
          tw[k + 2] = cos (4 * PI * i / N_p2);
          tw[k + 3] = sin (4 * PI * i / N_p2);
          tw[k + 4] = cos (6 * PI * i / N_p2);
          tw[k + 5] = sin (6 * PI * i / N_p2);
          k += 6;
      }
  }
  
  if ((s_r3 != 0) || (s_r5 != 0)) {
    /* generate twiddles for radix-3 loop */
    k = N_p2*2;
    if (dim == 2) k += 2*n;
    for (i = 0; i < s_r3; i++) {
      for (j = 0; j < n/3; j++) {
        tw[k]     = cos (2 * PI * j / n);
        tw[k + 1] = sin (2 * PI * j / n);
        tw[k + 2] = cos (4 * PI * j / n);
        tw[k + 3] = sin (4 * PI * j / n);
        k += 4;
      }
      n /= 3;
    }
    /* generate twiddles for radix-5 loop */
    for (i = 0; i < s_r5; i++) {
      for (j = 0; j < n/5; j++) {
        tw[k]     = cos (2 * PI * j / n);
        tw[k + 1] = sin (2 * PI * j / n);
        tw[k + 2] = cos (4 * PI * j / n);
        tw[k + 3] = sin (4 * PI * j / n);
        tw[k + 4] = cos (6 * PI * j / n);
        tw[k + 5] = sin (6 * PI * j / n);
        tw[k + 6] = cos (8 * PI * j / n);
        tw[k + 7] = sin (8 * PI * j / n);
        k += 8;
      }
      n /= 5;
    }
  }
} /* twiddle_gen()*/

fft_plan_t fft_omp_dp_plan_1d_c2c (
    int   N,        /* FFT size */
    int   mode,     /* FFT_DIRECT or FFT_ECPY */
    fft_callout_t fxns
)
{
    int          rad, n1, n2;
    int          int_tw_size, ext_tw_size, twsize;
    int          use_bs = 0;
    int          s_r3, s_r5, N_p2;

    double        *tw;

    fft_plan_t   p;

    /* Setup the fft function */
    p.fftfxn = fft_omp_dp_1d_c2c_ecpy;

    p.nbuf = FFT_DP_1D_C2C_NBUFS;

    memcpy(&p.fftcout, &fxns, sizeof(fft_callout_t));
    memcpy(p.bufs, FFT_dp_1d_c2c_Bufs, FFT_DP_1D_C2C_NBUFS*sizeof(FFTmemBuffer_t));

    /* determine the rad for the first  & second dimensions */
    calculate_rad(N, &n1, &n2, &p.u.dp_1d_c2c_e.para1, &p.u.dp_1d_c2c_e.para2, &use_bs);
    p.u.dp_1d_c2c_e.n1 = n1;
    p.u.dp_1d_c2c_e.n2 = n2;
    /* not supported, plan failed */
    if (use_bs == 1) {
      return (p);
    }
    
    /* calculate memory requirements */
    p.bufs[FFT_DP_1D_C2C_EXTERN_X_BUFN].size = sizeof(double)*2*N;
    p.bufs[FFT_DP_1D_C2C_EXTERN_Y_BUFN].size = sizeof(double)*2*N;
    p.bufs[FFT_DP_1D_C2C_INTERN_X_BUFN].size = sizeof(double)*4*n1*FFT_OMP_DP_1D_C2C_NUMOFLINEBUFS;
    p.bufs[FFT_DP_1D_C2C_INTERN_Y_BUFN].size = sizeof(double)*4*n1*FFT_OMP_DP_1D_C2C_NUMOFLINEBUFS;
    p.bufs[FFT_DP_1D_C2C_INTERN_T_BUFN].size = sizeof(double)*2*n1*FFT_OMP_DP_1D_C2C_NUMOFLINEBUFS;

    
    /* Calculate mem size for 2 dimensions for ECPY */
    /* evaluate the  1st dimension */
    s_r3 = p.u.dp_1d_c2c_e.para1.s_r3;
    s_r5 = p.u.dp_1d_c2c_e.para1.s_r5;
    N_p2 = p.u.dp_1d_c2c_e.para1.N_p2;
    calculate_mem_size(n1, s_r3, s_r5, N_p2, &twsize);
    p.u.dp_1d_c2c_e.para1.twsize = twsize;
    int_tw_size = ext_tw_size = twsize;
  
    /* evaluate the 2nd dimension */
    s_r3 = p.u.dp_1d_c2c_e.para2.s_r3;
    s_r5 = p.u.dp_1d_c2c_e.para2.s_r5;
    N_p2 = p.u.dp_1d_c2c_e.para2.N_p2;
    calculate_mem_size(n2, s_r3, s_r5, N_p2, &twsize);
    p.u.dp_1d_c2c_e.para2.twsize = twsize;
    int_tw_size  = (int_tw_size > twsize) ? int_tw_size : twsize;
    ext_tw_size += twsize + 2*n2;
    
    p.bufs[FFT_DP_1D_C2C_EXTERN_W_BUFN].size    = sizeof(double)*ext_tw_size;
    p.bufs[FFT_DP_1D_C2C_INTERN_W_BUFN].size    = sizeof(double)*int_tw_size;
    
    (*fxns.memoryRequest)(FFT_DP_1D_C2C_NBUFS, p.bufs);
    
    p.u.dp_1d_c2c_e.N      = N;
    p.u.dp_1d_c2c_e.data   = p.bufs[FFT_DP_1D_C2C_EXTERN_X_BUFN].base;
    p.u.dp_1d_c2c_e.work   = p.bufs[FFT_DP_1D_C2C_EXTERN_Y_BUFN].base;
    p.u.dp_1d_c2c_e.data_w = p.bufs[FFT_DP_1D_C2C_INTERN_X_BUFN].base;
    p.u.dp_1d_c2c_e.work_w = p.bufs[FFT_DP_1D_C2C_INTERN_Y_BUFN].base;
    p.u.dp_1d_c2c_e.work_t = p.bufs[FFT_DP_1D_C2C_INTERN_T_BUFN].base;
    p.u.dp_1d_c2c_e.tw_w   = p.bufs[FFT_DP_1D_C2C_INTERN_W_BUFN].base;
    tw                     = p.bufs[FFT_DP_1D_C2C_EXTERN_W_BUFN].base;
    p.u.dp_1d_c2c_e.tw     = tw;


    /* generate twiddle for the c2c fft*/
    s_r3 = p.u.dp_1d_c2c_e.para1.s_r3;
    s_r5 = p.u.dp_1d_c2c_e.para1.s_r5;
    N_p2 = p.u.dp_1d_c2c_e.para1.N_p2;
    rad  = p.u.dp_1d_c2c_e.para1.rad;
    twiddle_gen (1, N, n1, rad, s_r3, s_r5, N_p2, tw);
    
    twsize = p.u.dp_1d_c2c_e.para1.twsize;
    s_r3 = p.u.dp_1d_c2c_e.para2.s_r3;
    s_r5 = p.u.dp_1d_c2c_e.para2.s_r5;
    N_p2 = p.u.dp_1d_c2c_e.para2.N_p2;
    rad  = p.u.dp_1d_c2c_e.para2.rad;
    twiddle_gen (2, N, n2, rad, s_r3, s_r5, N_p2, tw+twsize);

    CACHE_wbInvAllL2(CACHE_WAIT);
    return (p);
}

/* ======================================================================== */
/*  End of file:  fft_omp_plan_dp_1d_c2c.c                                  */
/* ------------------------------------------------------------------------ */
/*            Copyright (c) 2013 Texas Instruments, Incorporated.           */
/*                           All Rights Reserved.                           */
/* ======================================================================== */
