/* ======================================================================= */
/*  TEXAS INSTRUMENTS, INC.                                                */
/*                                                                         */
/*  FFTLIB  FFT Library                                                    */
/*                                                                         */
/* Copyright (C) 2013 Texas Instruments Incorporated - http://www.ti.com/  */ 
/*                                                                         */
/*                                                                         */
/*  Redistribution and use in source and binary forms, with or without     */
/*  modification, are permitted provided that the following conditions     */
/*  are met:                                                               */
/*                                                                         */
/*    Redistributions of source code must retain the above copyright       */
/*    notice, this list of conditions and the following disclaimer.        */
/*                                                                         */
/*    Redistributions in binary form must reproduce the above copyright    */
/*    notice, this list of conditions and the following disclaimer in the  */
/*    documentation and/or other materials provided with the               */
/*    distribution.                                                        */
/*                                                                         */
/*    Neither the name of Texas Instruments Incorporated nor the names of  */
/*    its contributors may be used to endorse or promote products derived  */
/*    from this software without specific prior written permission.        */
/*                                                                         */
/*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS    */
/*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT      */
/*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR  */
/*  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT   */
/*  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,  */
/*  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT       */
/*  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,  */
/*  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY  */
/*  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT    */
/*  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE  */
/*  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.   */
/*                                                                         */
/* ======================================================================= */

#include <ti/fftlib/src/common/fft_common.h>

#ifndef FFT_OMP_DP_1D_C2C_H_
#define FFT_OMP_DP_1D_C2C_H_ 1

/** @ingroup 1D */
/* @{ */

/** @defgroup FFT_OMP_DP_1D_C2C */
/** @ingroup FFT_OMP_DP_1D_C2C */
/* @{ */

#define FFT_OMP_DP_1D_C2C_4STEP_MIN_SIZE  (     1024)
#define FFT_OMP_DP_1D_C2C_4STEP_MAX_SIZE  (1024*1024)

/**
 *     @param FFT_OMP_DP_1D_C2C_NUMOFLINEBUFS = number of cache lines to process each iteration
 */
 
#define FFT_OMP_DP_1D_C2C_NUMOFLINEBUFS 4

/**
 *
 *     @param N = FFT size dimension 1        
 *     @param mode = FFT_ECPY is supported for 1d multicore FFT
 */
 
/* FFT plan 1D DP API */
fft_plan_t fft_omp_dp_plan_1d_c2c (
    int   N,        /* FFT size */
    int   mode,     /* FFT_DIRECT or FFT_ECPY */
    fft_callout_t fxns
);

 /**
 *     This program performs one 1-dimensional double-precision multicore complex-to-complex FFT.
 *  
 *     @param N = FFT size      
 *     @param n1 = FFT size stage 1        
 *     @param n2 = FFT size stage 2
 *     @param para1 = Twiddle, mixed radix & bluestein parameters for stage 1
 *     @param para2 = Twiddle, mixed radix & bluestein parameters for stage 2
 *     @param data = input pointer    
 *     @param work = output pointer   
 *     @param data_w = input/output buffer pointer    
 *     @param work_w = working buffer pointer  
 *     @param work_t = Cache line buffer pointer  
 *     @param tw = twiddle pointer  
 *     @param tw_w = twiddle buffer pointer
 * 
 * @par Algorithm:
 *      Algorithm can run in radix 2, 3, 4 & 5 based upon value of N. <br><b>Note:</b>Parameters 
 *      <tt>para1</tt> & <tt>para2</tt> are determined at runtime and will depend upon type of FFT
 *      desired, radix 3 & 5, or radix 2 & 4.
 *
 * @par Assumptions:
 *      N1 & N2 are a multiple of FFT_OMP_DP_1D_C2C_NUMOFLINEBUFS and positive values.<br>
 *      All input, output and working buffers are double word aligned.<br>
 *      No input, working or buffers overlap.<br>
 *
 * @par Size Limitations:
 *      Current limitations are: 1024*1024 FFT size for power of 2. 
 *
 * @par Implementation notes:
 * @b Endian Support: The code supports little endian modes.  <BR> 
 * @b Interruptibility: The code is interrupt-tolerant but not interruptible. <BR>
 *  
 */
 
/* FFT execute 1D DP 4-step ecpy mode API */
int fft_omp_dp_1d_c2c_ecpy (
    /* int     N;       FFT size                         */
    /* int     n1;      stage1 FFT size                  */
    /* int     n2;      stage2 FFT size                  */
    /* double *data;    data pointer                     */
    /* double *work;    working buffer pointer           */
    /* double *data_w;  data internal pointer            */
    /* double *work_w;  working buffer internal pointer  */
    /* double *tw;      twiddle pointer                  */
    /* double *tw_w;    twiddle internal pointer         */
    fft_param_u u,
    void *edmaState
);

 /* }@ */ /* ingroup */
 /* }@ */ /* ingroup */
 
#endif /* FFT_OMP_DP_1D_C2C_H_ */

/* ======================================================================== */
/*  End of file:  fft_omp_dp_1d_c2c.h                                       */
/* ------------------------------------------------------------------------ */
/*            Copyright (c) 2013 Texas Instruments, Incorporated.           */
/*                           All Rights Reserved.                           */
/* ======================================================================== */


