/* ======================================================================= */
/*  TEXAS INSTRUMENTS, INC.                                                */
/*                                                                         */
/*  FFTLIB  FFT Library                                                    */
/*                                                                         */
/* Copyright (C) 2013 Texas Instruments Incorporated - http://www.ti.com/  */
/*                                                                         */
/*                                                                         */
/*  Redistribution and use in source and binary forms, with or without     */
/*  modification, are permitted provided that the following conditions     */
/*  are met:                                                               */
/*                                                                         */
/*    Redistributions of source code must retain the above copyright       */
/*    notice, this list of conditions and the following disclaimer.        */
/*                                                                         */
/*    Redistributions in binary form must reproduce the above copyright    */
/*    notice, this list of conditions and the following disclaimer in the  */
/*    documentation and/or other materials provided with the               */
/*    distribution.                                                        */
/*                                                                         */
/*    Neither the name of Texas Instruments Incorporated nor the names of  */
/*    its contributors may be used to endorse or promote products derived  */
/*    from this software without specific prior written permission.        */
/*                                                                         */
/*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS    */
/*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT      */
/*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR  */
/*  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT   */
/*  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,  */
/*  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT       */
/*  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,  */
/*  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY  */
/*  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT    */
/*  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE  */
/*  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.   */
/*                                                                         */
/* ======================================================================= */

#include <math.h>
#include <stdio.h>
#include <c6x.h>
#include "ifft_omp_sp_3d_c2r.h"
#include <ti/fftlib/src/common/fft/fft_sp_cmplx_notwid_br.h>
#include <ti/fftlib/src/common/ifft/ifft_sp_cmplx_notwid_br.h>
#include <ti/csl/csl_cacheAux.h>

/* IFFT 3d sp c2r memory buffers */
#define IFFT_SP_3D_C2R_EXTERN_X_BUFN      0
#define IFFT_SP_3D_C2R_EXTERN_Y_BUFN      (IFFT_SP_3D_C2R_EXTERN_X_BUFN  +1)
#define IFFT_SP_3D_C2R_EXTERN_W_BUFN      (IFFT_SP_3D_C2R_EXTERN_Y_BUFN  +1)
#define IFFT_SP_3D_C2R_EXTERN_BN_BUFN     (IFFT_SP_3D_C2R_EXTERN_W_BUFN  +1)
#define IFFT_SP_3D_C2R_INTERN_X_BUFN      (IFFT_SP_3D_C2R_EXTERN_BN_BUFN  +1)
#define IFFT_SP_3D_C2R_INTERN_Y_BUFN      (IFFT_SP_3D_C2R_INTERN_X_BUFN  +1)
#define IFFT_SP_3D_C2R_INTERN_W_BUFN      (IFFT_SP_3D_C2R_INTERN_Y_BUFN  +1)
#define IFFT_SP_3D_C2R_INTERN_BN_BUFN     (IFFT_SP_3D_C2R_INTERN_W_BUFN  +1)
#define IFFT_SP_3D_C2R_INTERN_WORK_BUFN   (IFFT_SP_3D_C2R_INTERN_BN_BUFN +1)
#define IFFT_SP_3D_C2R_INTERN_T_BUFN      (IFFT_SP_3D_C2R_INTERN_WORK_BUFN +1)
#define IFFT_SP_3D_C2R_NBUFS              (IFFT_SP_3D_C2R_INTERN_T_BUFN  +1)

/* class, log2align, size, base */
static FFTmemBuffer_t IFFT_sp_3d_c2r_Bufs[IFFT_SP_3D_C2R_NBUFS] = {
  {FFT_MEM_CLASS_EXTERNAL, 3, 0, 0},   /* EXTERN_X */
  {FFT_MEM_CLASS_EXTERNAL, 3, 0, 0},   /* EXTERN_Y */
  {FFT_MEM_CLASS_EXTERNAL, 3, 0, 0},   /* EXTERN_W */
  {FFT_MEM_CLASS_EXTERNAL, 3, 0, 0},   /* EXTERN_BN */
  {FFT_MEM_CLASS_INTERNAL, 3, 0, 0},   /* INTERN_X */
  {FFT_MEM_CLASS_INTERNAL, 3, 0, 0},   /* INTERN_Y */
  {FFT_MEM_CLASS_INTERNAL, 3, 0, 0},   /* INTERN_W */
  {FFT_MEM_CLASS_INTERNAL, 3, 0, 0},   /* INTERN_BN */
  {FFT_MEM_CLASS_INTERNAL, 3, 0, 0},   /* INTERN_WORK */
  {FFT_MEM_CLASS_INTERNAL, 3, 0, 0}    /* INTERN_T */
};

/* calculate radix for 1D array */
static void calculate_rad (int dim, int N, fft_para_mix_bs_t *fft_para, int *use_bs)
{
  int n, j, M, rad;
  int s_r3, s_r5, N_p2, dum;

  *use_bs = 1;

  /* calculate the size for zero padding when the signal size is not power of 2 */
  n = (N<<1)-1;
  j = _norm(n);
  M = 1 << (31-j);

  if (dim==3)
    N_p2 = N/2;
  else
    N_p2 = N;
  s_r3 = 0;
  s_r5 = 0;

  /* if N is odd, bypass the rest of calculation, use Bluestein */
  if (((N>>1)<<1) == N)
  {
    /* if N is power of 2, no zero padding, regular radix 2 or 4 fft */
    if (N == (M>>1))
    { /* power of 2 */
      M = N;
      j = j+1;
      *use_bs = 0;
    }
    else {
      if (dim==3)
        dum = N/2;
      else
        dum = N;
      while (dum/3*3 == dum) {
        s_r3++;
        dum /= 3;
      }
      while (dum/5*5 == dum) {
        s_r5++;
        dum /= 5;
      }
      if ((dum >= 8) && (dum == (1 << (30-_norm(dum))))) {
        /* mixed radix 2 and 3 */
        N_p2                 = dum;
        *use_bs = 0;
        M = N;
        j  = _norm(dum);
      }
    }
  }

  /* calculate radix for radix 2 or 4 c2r fft */
  if (dim==3)
  {
    if (j%2 == 1)
      rad = 4;
    else
      rad = 2;
  } else
  {
	if (j%2 == 0)
	  rad = 4;
	else
	  rad = 2;
  }

  fft_para->N_p2 = N_p2;
  fft_para->s_r3 = s_r3;
  fft_para->s_r5 = s_r5;
  fft_para->M    = M;
  fft_para->rad  = rad;

} /*calculate_rad ()*/

/* calculate twiddle size, Bn buffer size and workbuffer size */
static void calculate_mem_size(int dim, int N, int M, int use_bs, int s_r3, int s_r5, int N_p2, int *twsize, int *Bnsize, int *worksize)
{
  int dum, n, i;
  /* Please note that Bn buffer and Workbuf are only used when doing Bluestein
   * meaning when use_bs == 1 */

  if (use_bs == 1)
  { /* when size is not power of 2 and not multiple of 3&5 */
    *twsize   = (2+2*N+2);
    *Bnsize   = (2*M);
    *worksize = (4*M);
  }
  else
  {
    if ((s_r3 == 0) && (s_r5 == 0)) {
    /* When signal is power of 2 */
      if (dim==3)
        *twsize = N/2;
      else
        *twsize = 2;
    }
    else {
    /* When signal is power of 3 or 5 */
      dum = 0;
      if (dim==3)
        n  = 2*N;
      else
        n  = 4*N;
      for (i = 0; i < s_r3; i++) {
        dum += n/3;
        n  /= 3;
      }
      n *= 2;  /* radix-5 has 8 twiddle vs radix-3 4 twiddles */
      for (i = 0; i < s_r5; i++) {
        dum += n/5;
        n  /= 5;
      }
      if (dim==3)
        *twsize = (N/2+dum);
      else
        *twsize = (2+dum);
    }
    *Bnsize   = 0;
    *worksize = 0;
  }
} /* calculate_mem_size() */

/* twiddle generation */
static void twiddle_gen (int dim, int N, int M, int rad, int use_bs, int s_r3, int s_r5, int N_p2, float *tw, float *Bn, float *work)
{
  int     k, n, j, i;
  float *bn, *Bk;
  const double PI = 3.14159265358979323846;

  k = 0;
  if (use_bs == 0){
    tw[k    ] =  sin (2 * PI / N_p2);
    tw[k + 1] =  cos (2 * PI / N_p2);

    if (dim==3)
    {
      k = 2;
      for (i = 1; i < N >> 2; i++) {
  #ifdef _LITTLE_ENDIAN
        tw[k    ] =  sin (2 * PI * i / N);
        tw[k + 1] = -cos (2 * PI * i / N);
  #else
        tw[k    ] =  sin (2 * PI * i / N);
        tw[k + 1] = -cos (2 * PI * i / N);
  #endif
        k += 2;
      }
    }

    if ((s_r3 != 0) || (s_r5 != 0)) {
      /* generate twiddles for radix-3 loop */
      if (dim==3)
      {
        k  = N/2;
        n = N/2;
      } else
      {
        k  = 2;
        n = N;
      }
      for (i = 0; i < s_r3; i++) {
          for (j = 0; j < n/3; j++) {
#ifdef _LITTLE_ENDIAN
            tw[k]     = (float) (-sin (2 * PI * j / n) / 3);
            tw[k + 1] = (float) ( cos (2 * PI * j / n) / 3);
            tw[k + 2] = (float) (-sin (4 * PI * j / n) / 3);
            tw[k + 3] = (float) ( cos (4 * PI * j / n) / 3);
#else
            tw[k]     = (float) ( cos (2 * PI * j / n) / 3);
            tw[k + 1] = (float) ( sin (2 * PI * j / n) / 3);
            tw[k + 2] = (float) ( cos (4 * PI * j / n) / 3);
            tw[k + 3] = (float) ( sin (4 * PI * j / n) / 3);
#endif
            k += 4;
          }
          n /= 3;
        }

        /* generate twiddles for radix-5 loop */
        for (i = 0; i < s_r5; i++) {
          for (j = 0; j < n/5; j++) {
#ifdef _LITTLE_ENDIAN
            tw[k]     = (float) (-sin (2 * PI * j / n) / 5);
            tw[k + 1] = (float) ( cos (2 * PI * j / n) / 5);
            tw[k + 2] = (float) (-sin (4 * PI * j / n) / 5);
            tw[k + 3] = (float) ( cos (4 * PI * j / n) / 5);
            tw[k + 4] = (float) (-sin (6 * PI * j / n) / 5);
            tw[k + 5] = (float) ( cos (6 * PI * j / n) / 5);
            tw[k + 6] = (float) (-sin (8 * PI * j / n) / 5);
            tw[k + 7] = (float) ( cos (8 * PI * j / n) / 5);
#else
            tw[k]     = (float) (cos (2 * PI * j / n) / 5);
            tw[k + 1] = (float) (sin (2 * PI * j / n) / 5);
            tw[k + 2] = (float) (cos (4 * PI * j / n) / 5);
            tw[k + 3] = (float) (sin (4 * PI * j / n) / 5);
            tw[k + 4] = (float) (cos (6 * PI * j / n) / 5);
            tw[k + 5] = (float) (sin (6 * PI * j / n) / 5);
            tw[k + 6] = (float) (cos (8 * PI * j / n) / 5);
            tw[k + 7] = (float) (sin (8 * PI * j / n) / 5);
#endif
            k += 8;
          }
          n /= 5;
        }
      }
    }
    else
    { /* generate twiddle for c2r fft */

      /*--------------------------------------
       *  TW is organized as follows:
       *     twiddle for fft,  size: 2
       *     Bk,               size: 2*N
       *     twiddle for ifft, size: 2
       *-------------------------------------*/

      tw[k    ] = -sin (2 * PI / M);
      tw[k + 1] =  cos (2 * PI / M);

      k = 2;

      /* phase factor generation */
      for (i = 0; i < N ; i++) {
        tw[k]     = (float)  cos (PI * i * i / N);
        tw[k + 1] = (float) -sin (PI * i * i / N);

        k += 2;
      }

      k = 2*N + 2;

      /* twiddle for ifft */
      tw[k    ] =  sin (2 * PI / M);
      tw[k + 1] =  cos (2 * PI / M);

      /* precalculate FFT of bn */

#pragma MUST_ITERATE(4,,4)
      /* first reset the work buffer to zeros */
      for (i = 0; i<M; i+=1)
      {
         _amem8_f2(&work[2*i]) = 0;
      }

      /* construct bn in work buffer */
      bn = work;
      Bk = tw + 2;

      /*---------------------------------------------*
       *         bn(0)    = Bk(0)
       *         bn(i)    = Bk(i)    0 < i <N
       *         bn(N1-i) = Bk(i)    N <= i <N1
       *  Note: bn(i) = Re(bn(i)) + i* Im(bn(i));
       ------------------------------------------- */
      bn[0] = Bk[0]/N;
      bn[1] = Bk[1]/N;

      for(i = 1; i < N; i++)
      {
        bn[2*i]   = bn[2*(M-i)]   = Bk[2*i]/N;
        bn[2*i+1] = bn[2*(M-i)+1] = Bk[2*i+1]/N;
      }

      /* FFT of the constructed signal Bn = fft(bn) */
      fft_sp_cmplx_notwid_br((double*)bn, M, (double*)Bn, (double*)tw);
    }
} /* twiddle_gen() */

fft_plan_t ifft_omp_sp_plan_3d_c2r (
    int   N1,       /* FFT size */
    int   N2,       /* FFT size */
    int   N3,       /* FFT size */
    int   mode,     /* FFT_DIRECT or FFT_ECPY */
    fft_callout_t fxns
)
{
    int          N, flag = 1;
    int          M1, M2, M3, rad;
    int          int_tw_size, ext_tw_size, int_Bn_size, ext_Bn_size, work_size;
    int          twsize, Bnsize, worksize;
    int          use_bs_1 = 0, use_bs_2 = 0, use_bs_3 = 0;
    int          s_r3, s_r5, N_p2;
    float        *tw, *tw_w;
    float        *restrict Bn, *restrict work;
    fft_plan_t   p;

    /* only support FFT_ECPY */
    p.fftfxn = ifft_omp_sp_3d_c2r_ecpy;

    /* choose the biggest size of all the dimensions */
    N = (N1 > N2) ? N1 : N2;
    N = (N3 > N) ? N3 : N;

    p.nbuf = IFFT_SP_3D_C2R_NBUFS;

    memcpy(&p.fftcout, &fxns, sizeof(fft_callout_t));
    memcpy(p.bufs, IFFT_sp_3d_c2r_Bufs, IFFT_SP_3D_C2R_NBUFS*sizeof(FFTmemBuffer_t));

    p.u.sp_3d_c2r_e.N1     = N1;
    p.u.sp_3d_c2r_e.N2     = N2;
    p.u.sp_3d_c2r_e.N3     = N3;

    /* determine the radix for the first dimension */
    calculate_rad(1, N1, &p.u.sp_3d_c2r_e.para1, &use_bs_1);
    M1 = p.u.sp_3d_c2r_e.para1.M;

    /* determine the radix for the second dimension */
    calculate_rad(2, N2, &p.u.sp_3d_c2r_e.para2, &use_bs_2);
    M2 = p.u.sp_3d_c2r_e.para2.M;

    /* determine the radix for the third dimension */
    calculate_rad(3, N3, &p.u.sp_3d_c2r_e.para3, &use_bs_3);
    M3 = p.u.sp_3d_c2r_e.para3.M;

    if ((N1*N3*2*sizeof(float)>= 32768) || (N1*N2*2*sizeof(float))>= 32768)
    	flag = 0;

    p.u.sp_3d_c2r_e.ecpy_flag  = flag;

    /* calculate memory requirements */
    p.bufs[IFFT_SP_3D_C2R_EXTERN_X_BUFN].size = sizeof(float)*N1*N2*N3*2;
    p.bufs[IFFT_SP_3D_C2R_EXTERN_Y_BUFN].size = sizeof(float)*N1*N2*N3*2;
    p.bufs[IFFT_SP_3D_C2R_EXTERN_W_BUFN].size = sizeof(float)*6;
    p.bufs[IFFT_SP_3D_C2R_INTERN_X_BUFN].size = sizeof(float)*N*4*IFFT_OMP_SP_3D_C2R_NUMOFLINEBUFS;
    p.bufs[IFFT_SP_3D_C2R_INTERN_Y_BUFN].size = sizeof(float)*N*4*IFFT_OMP_SP_3D_C2R_NUMOFLINEBUFS;
    p.bufs[IFFT_SP_3D_C2R_INTERN_T_BUFN].size = sizeof(float)*N*4*IFFT_OMP_SP_3D_C2R_NUMOFLINEBUFS;
    p.bufs[IFFT_SP_3D_C2R_INTERN_W_BUFN].size = sizeof(float)*6;

    /* evaluate the 1st dimension */
    s_r3 = p.u.sp_3d_c2r_e.para1.s_r3;
    s_r5 = p.u.sp_3d_c2r_e.para1.s_r5;
    N_p2 = p.u.sp_3d_c2r_e.para1.N_p2;
    calculate_mem_size(1, N1, M1, use_bs_1, s_r3, s_r5, N_p2, &twsize, &Bnsize, &worksize);
    p.u.sp_3d_c2r_e.para1.twsize = twsize;
    p.u.sp_3d_c2r_e.para1.Bnsize = Bnsize;
    int_tw_size = ext_tw_size = twsize;
    int_Bn_size = ext_Bn_size = Bnsize;
    work_size   = worksize;

    /* evaluate the  2nd dimension */
    s_r3 = p.u.sp_3d_c2r_e.para2.s_r3;
    s_r5 = p.u.sp_3d_c2r_e.para2.s_r5;
    N_p2 = p.u.sp_3d_c2r_e.para2.N_p2;
    calculate_mem_size(2, N2, M2, use_bs_2, s_r3, s_r5, N_p2, &twsize, &Bnsize, &worksize);
    p.u.sp_3d_c2r_e.para2.twsize = twsize;
    p.u.sp_3d_c2r_e.para2.Bnsize = Bnsize;
    int_tw_size  = (int_tw_size > twsize) ? int_tw_size : twsize;
    ext_tw_size += twsize;
    int_Bn_size  = (int_Bn_size > Bnsize) ? int_Bn_size : Bnsize;
    ext_Bn_size += Bnsize;
    work_size    = (work_size > worksize) ? work_size : worksize;

    /* evaluate the  3rd dimension */
    s_r3 = p.u.sp_3d_c2r_e.para3.s_r3;
    s_r5 = p.u.sp_3d_c2r_e.para3.s_r5;
    N_p2 = p.u.sp_3d_c2r_e.para3.N_p2;
    calculate_mem_size(3, N3, M3, use_bs_3, s_r3, s_r5, N_p2, &twsize, &Bnsize, &worksize);
    p.u.sp_3d_c2r_e.para3.twsize = twsize;
    p.u.sp_3d_c2r_e.para3.Bnsize = Bnsize;
    int_tw_size  = (int_tw_size > twsize) ? int_tw_size : twsize;
    ext_tw_size += twsize;
    int_Bn_size  = (int_Bn_size > Bnsize) ? int_Bn_size : Bnsize;
    ext_Bn_size += Bnsize;
    work_size    = (work_size > worksize) ? work_size : worksize;

    p.bufs[IFFT_SP_3D_C2R_EXTERN_W_BUFN].size    = sizeof(float)*ext_tw_size;
    p.bufs[IFFT_SP_3D_C2R_INTERN_W_BUFN].size    = sizeof(float)*int_tw_size;
    p.bufs[IFFT_SP_3D_C2R_EXTERN_BN_BUFN].size   = sizeof(float)*ext_Bn_size;
    p.bufs[IFFT_SP_3D_C2R_INTERN_BN_BUFN].size   = sizeof(float)*int_Bn_size;
    p.bufs[IFFT_SP_3D_C2R_INTERN_WORK_BUFN].size = sizeof(float)*work_size;

    (*fxns.memoryRequest)(IFFT_SP_3D_C2R_NBUFS, p.bufs);

    p.u.sp_3d_c2r_e.in    = p.bufs[IFFT_SP_3D_C2R_EXTERN_X_BUFN].base;
    p.u.sp_3d_c2r_e.out   = p.bufs[IFFT_SP_3D_C2R_EXTERN_Y_BUFN].base;
    p.u.sp_3d_c2r_e.in_w  = p.bufs[IFFT_SP_3D_C2R_INTERN_X_BUFN].base;
    p.u.sp_3d_c2r_e.out_w = p.bufs[IFFT_SP_3D_C2R_INTERN_Y_BUFN].base;
    p.u.sp_3d_c2r_e.out_t = p.bufs[IFFT_SP_3D_C2R_INTERN_T_BUFN].base;
    tw                    = p.bufs[IFFT_SP_3D_C2R_EXTERN_W_BUFN].base;
    tw_w                  = p.bufs[IFFT_SP_3D_C2R_INTERN_W_BUFN].base;
    p.u.sp_3d_c2r_e.tw    = tw;
    p.u.sp_3d_c2r_e.tw_w  = tw_w;

    if ((use_bs_1 == 1) || (use_bs_2 == 1) || (use_bs_3 == 1))
    {
      /* if use Bluestein FFT, set the buffers to do Bluestein */
      p.u.sp_3d_c2r_e.Bn         = p.bufs[IFFT_SP_3D_C2R_EXTERN_BN_BUFN].base;
      p.u.sp_3d_c2r_e.Bn_w       = p.bufs[IFFT_SP_3D_C2R_INTERN_BN_BUFN].base;
      p.u.sp_3d_c2r_e.workbuf_bn = p.bufs[IFFT_SP_3D_C2R_INTERN_WORK_BUFN].base;
    }
    else
    {
      /* otherwise, set the buffers to NULL */
      p.u.sp_3d_c2r_e.Bn         = NULL;
      p.u.sp_3d_c2r_e.Bn_w       = NULL;
      p.u.sp_3d_c2r_e.workbuf_bn = NULL;
    }
    
    /* twiddle generation */
    Bn   = p.u.sp_3d_c2r_e.Bn;
    work = p.u.sp_3d_c2r_e.workbuf_bn;

    /* generate twiddle for the first dimension c2r ifft */
    /* split factors */
    s_r3 = p.u.sp_3d_c2r_e.para1.s_r3;
    s_r5 = p.u.sp_3d_c2r_e.para1.s_r5;
    N_p2 = p.u.sp_3d_c2r_e.para1.N_p2;
    rad  = p.u.sp_3d_c2r_e.para1.rad;
    twiddle_gen (1, N1, M1, rad, use_bs_1, s_r3, s_r5, N_p2, tw, Bn, work);

    /* generate twiddle for the second dimension c2r ifft */
    twsize = p.u.sp_3d_c2r_e.para1.twsize;
    Bnsize = p.u.sp_3d_c2r_e.para1.Bnsize;

    s_r3 = p.u.sp_3d_c2r_e.para2.s_r3;
    s_r5 = p.u.sp_3d_c2r_e.para2.s_r5;
    N_p2 = p.u.sp_3d_c2r_e.para2.N_p2;
    rad  = p.u.sp_3d_c2r_e.para2.rad;
    twiddle_gen (2, N2, M2, rad, use_bs_2, s_r3, s_r5, N_p2, tw+twsize, Bn+Bnsize, work);

    /* generate twiddle for the third dimension c2r ifft */
    twsize = p.u.sp_3d_c2r_e.para1.twsize + p.u.sp_3d_c2r_e.para2.twsize;
    Bnsize = p.u.sp_3d_c2r_e.para1.Bnsize + p.u.sp_3d_c2r_e.para2.Bnsize;

    s_r3 = p.u.sp_3d_c2r_e.para3.s_r3;
    s_r5 = p.u.sp_3d_c2r_e.para3.s_r5;
    N_p2 = p.u.sp_3d_c2r_e.para3.N_p2;
    rad  = p.u.sp_3d_c2r_e.para3.rad;
    twiddle_gen (3, N3, M3, rad, use_bs_3, s_r3, s_r5, N_p2, tw+twsize, Bn+Bnsize, work);

    CACHE_wbInvAllL2(CACHE_WAIT);
    return (p);
}

/* ======================================================================== */
/*  End of file:  ifft_omp_plan_sp_3d_c2r.c                                     */
/* ------------------------------------------------------------------------ */
/*            Copyright (c) 2013 Texas Instruments, Incorporated.           */
/*                           All Rights Reserved.                           */
/* ======================================================================== */
